#! /usr/bin/perl


use strict;

use IO::File;
use IO::Pipe;
use File::Copy;

use gateway_lib;

chdir("/tmp");

my $TMCC = "/usr/local/etc/emulab/tmcc";
my $tmcc_p = new IO::Pipe() || die "Can't open pipe: $!\n";
my $shared_config_dir;
my $local_config_dir = "/usr/local/federation/etc";
my %services;
my $perl;

my $gateway;
my $smbshare = "USERS";
my $smbuser;
my $smbproject;
chomp (my $uname = `uname`);
my $smbmount = "smbmount.$uname.pl";

# find perl
for my $p ("/usr/bin/perl", "/usr/local/bin/perl") {
    if ( -x $p ) {
	$perl = $p;
	last;
    }
}
$perl = "perl" unless $perl;


$tmcc_p->reader("$TMCC -b status");
while (<$tmcc_p>) {
    /ALLOCATED=([^\/]+)\/(\S+)/ && do {
	$shared_config_dir = "/proj/$1/exp/$2/tmp";
	last;
    };
}
$tmcc_p->close();

mkdir($local_config_dir);

foreach my $fn ("seer.conf", "client.conf", "userconf") {
    copy("$shared_config_dir/$fn", $local_config_dir )
	if -e "$shared_config_dir/$fn";
}

my $client = new IO::File("$local_config_dir/client.conf");
while (<$client>) {
    chomp;
    /ControlGateway:\s+(.*)/i && do { $gateway = $1; };
    /SMBShare:\s+(.*)/i && do { $smbshare = $1; };
    /ProjectUser:\s+(.*)/i && do { $smbuser = $1; };
    /ProjectName:\s+(.*)/i && do { $smbproject = $1; };
    /Service:\s+(.*)/i && do { $services{$1}++;};
}
$client->close();
# Create the /etc/hosts file
my $hosts = new IO::File("/etc/hosts") || die "Can't open /etc/hosts:$!\n";
my $new_hosts = new IO::File(">/tmp/hosts") || die "Can't open /tmp/hosts:$!\n";
my $config_hosts = new IO::File("$shared_config_dir/hosts") || 
    die "Can't open $shared_config_dir/hosts: $!\n";

while (<$hosts>) {
    /^127\.0\.0\.1/ && do { print $new_hosts $_; };
}
$hosts->close();
while (<$config_hosts>) {
    print $new_hosts $_;
}
$config_hosts->close();
$new_hosts->close();
copy("/tmp/hosts", "/etc/hosts");


if ($services{'userconfig'}) {
    $tmcc_p = new IO::Pipe() || die "Can't open pipe for accounts:$!\n";
    my $old_accounts = new IO::File(">$local_config_dir/old_accts") || 
	die "Can't open $local_config_dir/old_accts: $!\n";

    $tmcc_p->reader("$TMCC -b accounts");
    while (<$tmcc_p>) {
	print $old_accounts $_;
    }
    $tmcc_p->close();
    $old_accounts->close();
    print("Updating accounts");
    system("/usr/local/federation/bin/rc.fedaccounts");
}

if ($services{'SMB'}) {
    if ($uname =~ /FreeBSD/ ) {
	system("umount -A -f -t nfs,smbfs,cifs");
	# Restart ntp
	system("/etc/rc.d/ntpd stop; /usr/sbin/ntpdate boss; " . 
	    "/etc/rc.d/ntpd start;");
						   
    }
    elsif ($uname =~ /Linux/ ) {
	# Pass individual filestems to Linux umount.  No -A.
	my $mtab = new IO::File("/etc/mtab") || die "Can't open /etc/mtab:$1\n";
	while (<$mtab>) {
	    chomp;
	    my @F = split($_);
	    next unless $F[2] =~ /(nfs|cifs|smbfs)/;
	    system("umount -f $F[1]");
	}
	# restart ntp
	system("/etc/rc.d/init.d/ntpd stop; /usr/sbin/ntpdate boss; ". 
	    "/etc/rc.d/init.d/ntpd start");
    }

    print "Waiting for SMB server\n";
    gateway_lib::wait_for_port($gateway, 139, 60*60) || 
	die "SMB server never came up\n";
    print "Mounting via SMB\n";
    system("$perl /usr/local/federation/bin/$smbmount $smbshare $gateway " . 
	"$smbuser $smbproject");
}

# startcmd
if ($ARGV[0] && $ARGV[1]) {
    if ($uname =~ /FreeBSD/) {
	system("su -l \"$ARGV[0]\" -c \"$ARGV[1]\"");
    }
    elsif ($uname =~ /Linux/) {
	system("su \"$ARGV[0]\" --command \"$ARGV[1]\"");
    }
}
exit(0);
